@@ -19,13 +19,13 @@ local nuts               = nodes.nuts
 local tonode             = nuts.tonode
 local tonut              = nuts.tonut
 
-local getfield           = nuts.getfield
 local getnext            = nuts.getnext
 local getprev            = nuts.getprev
 local getid              = nuts.getid
 local getlist            = nuts.getlist
 local getleader          = nuts.getleader
 local getattr            = nuts.getattr
+local getwidth           = nuts.getwidth
 
 local setlist            = nuts.setlist
 local setleader          = nuts.setleader
@@ -42,12 +42,9 @@ local glyph_code         = nodecodes.glyph
 local disc_code          = nodecodes.disc
 local glue_code          = nodecodes.glue
 local rule_code          = nodecodes.rule
-local whatsit_code       = nodecodes.whatsit
 local hlist_code         = nodecodes.hlist
 local vlist_code         = nodecodes.vlist
 
-local normal_rule        = rulecodes.normal
-
 local states             = attributes.states
 local numbers            = attributes.numbers
 local a_trigger          = attributes.private('trigger')
@@ -126,12 +123,6 @@ function nodes.installattributehandler(plugin)
     return loadstripped(template)()
 end
 
--- for the moment:
-
-local function copied(n)
-    return copy_node(tonut(n))
-end
-
 -- the injectors
 
 local nsdata, nsnone, nslistwise, nsforced, nsselector, nstrigger
@@ -165,13 +156,13 @@ function states.finalize(namespace,attribute,head) -- is this one ok?
         if id == hlist_code or id == vlist_code then
             local content = getlist(head)
             if content then
-                local list = insert_node_before(content,content,copied(nsnone)) -- two return values
+                local list = insert_node_before(content,content,copy_node(nsnone)) -- two return values
                 if list ~= content then
                     setlist(head,list)
                 end
             end
         else
-            head = insert_node_before(head,head,copied(nsnone))
+            head = insert_node_before(head,head,copy_node(nsnone))
         end
         return tonode(head), true, true
     end
@@ -179,19 +170,16 @@ function states.finalize(namespace,attribute,head) -- is this one ok?
 end
 
 -- we need to deal with literals too (reset as well as oval)
--- if id == glyph_code or (id == whatsit_code and getsubtype(stack) == pdfliteral_code) or (id == rule_code and stack.width ~= 0) or (id == glue_code and stack.leader) then
 
-local function process(namespace,attribute,head,inheritance,default) -- one attribute
+local function process(attribute,head,inheritance,default) -- one attribute
     local stack  = head
     local done   = false
     local check  = false
     local leader = nil
     while stack do
         local id = getid(stack)
-        if id == glyph_code then
-            check = true
-        elseif id == disc_code then
-            check = true -- no longer needed as we flatten replace
+        if id == glyph_code or id == disc_code then
+            check = true -- disc no longer needed as we flatten replace
         elseif id == glue_code then
             leader = getleader(stack)
             if leader then
@@ -204,31 +192,35 @@ local function process(namespace,attribute,head,inheritance,default) -- one attr
                 if nstrigger and getattr(stack,nstrigger) then
                     local outer = getattr(stack,attribute)
                     if outer ~= inheritance then
-                        local list, ok = process(namespace,attribute,content,inheritance,outer)
+                        local list, ok = process(attribute,content,inheritance,outer)
                         if content ~= list then
                             setlist(stack,list)
                         end
-                        done = done or ok
+                        if ok then
+                            done = true
+                        end
                     else
-                        local list, ok = process(namespace,attribute,content,inheritance,default)
+                        local list, ok = process(attribute,content,inheritance,default)
                         if content ~= list then
                             setlist(stack,list)
                         end
-                        done = done or ok
+                        if ok then
+                            done = true
+                        end
                     end
                 else
-                    local list, ok = process(namespace,attribute,content,inheritance,default)
+                    local list, ok = process(attribute,content,inheritance,default)
                     if content ~= list then
                         setlist(stack,list)
                     end
-                    done = done or ok
+                    if ok then
+                        done = true
+                    end
                 end
                 -- end nested --
             end
         elseif id == rule_code then
--- if subtype(stack) == normal_rule then
-            check = getfield(stack,"width") ~= 0
--- end
+            check = getwidth(stack) ~= 0
         end
         -- much faster this way than using a check() and nested() function
         if check then
@@ -236,14 +228,14 @@ local function process(namespace,attribute,head,inheritance,default) -- one attr
             if c then
                 if default and c == inheritance then
                     if current ~= default then
-                        head = insert_node_before(head,stack,copied(nsdata[default]))
+                        head    = insert_node_before(head,stack,copy_node(nsdata[default]))
                         current = default
-                        done = true
+                        done    = true
                     end
                 elseif current ~= c then
-                    head = insert_node_before(head,stack,copied(nsdata[c]))
+                    head    = insert_node_before(head,stack,copy_node(nsdata[c]))
                     current = c
-                    done = true
+                    done    = true
                 end
                 if leader then
                     local savedcurrent = current
@@ -258,24 +250,30 @@ local function process(namespace,attribute,head,inheritance,default) -- one attr
                     if nstrigger and getattr(stack,nstrigger) then
                         local outer = getattr(stack,attribute)
                         if outer ~= inheritance then
-                            local list, ok = process(namespace,attribute,leader,inheritance,outer)
+                            local list, ok = process(attribute,leader,inheritance,outer)
                             if leader ~= list then
                                 setleader(stack,list)
                             end
-                            done = done or ok
+                            if ok then
+                                done = true
+                            end
                         else
-                            local list, ok = process(namespace,attribute,leader,inheritance,default)
+                            local list, ok = process(attribute,leader,inheritance,default)
                             if leader ~= list then
                                 setleader(stack,list)
                             end
-                            done = done or ok
+                            if ok then
+                                done = true
+                            end
                         end
                     else
-                        local list, ok = process(namespace,attribute,leader,inheritance,default)
+                        local list, ok = process(attribute,leader,inheritance,default)
                         if leader ~= list then
                             setleader(stack,list)
                         end
-                        done = done or ok
+                        if ok then
+                            done = true
+                        end
                     end
                     -- end nested --
                     current = savedcurrent
@@ -283,14 +281,14 @@ local function process(namespace,attribute,head,inheritance,default) -- one attr
                 end
             elseif default and inheritance then
                 if current ~= default then
-                    head = insert_node_before(head,stack,copied(nsdata[default]))
+                    head    = insert_node_before(head,stack,copy_node(nsdata[default]))
                     current = default
-                    done = true
+                    done    = true
                 end
             elseif current > 0 then
-                head = insert_node_before(head,stack,copied(nsnone))
+                head    = insert_node_before(head,stack,copy_node(nsnone))
                 current = 0
-                done = true
+                done    = true
             end
             check = false
         end
@@ -300,7 +298,7 @@ local function process(namespace,attribute,head,inheritance,default) -- one attr
 end
 
 states.process = function(namespace,attribute,head,default)
-    local head, done = process(namespace,attribute,tonut(head),default)
+    local head, done = process(attribute,tonut(head),default)
     return tonode(head), done
 end
 
@@ -310,17 +308,16 @@ end
 -- state changes while the main state stays the same (like two glyphs following
 -- each other with the same color but different color spaces e.g. \showcolor)
 
-local function selective(namespace,attribute,head,inheritance,default) -- two attributes
+local function selective(attribute,head,inheritance,default) -- two attributes
+ -- local head   = head
     local stack  = head
     local done   = false
     local check  = false
     local leader = nil
     while stack do
         local id = getid(stack)
-        if id == glyph_code then
-            check = true
-        elseif id == disc_code then
-            check = true -- notneeded when we flatten replace
+        if id == glyph_code or id == disc_code then
+            check = true -- disc no longer needed as we flatten replace
         elseif id == glue_code then
             leader = getleader(stack)
             if leader then
@@ -333,31 +330,35 @@ local function selective(namespace,attribute,head,inheritance,default) -- two at
                 if nstrigger and getattr(stack,nstrigger) then
                     local outer = getattr(stack,attribute)
                     if outer ~= inheritance then
-                        local list, ok = selective(namespace,attribute,content,inheritance,outer)
+                        local list, ok = selective(attribute,content,inheritance,outer)
                         if content ~= list then
                             setlist(stack,list)
                         end
-                        done = done or ok
+                        if ok then
+                            done = true
+                        end
                     else
-                        local list, ok = selective(namespace,attribute,content,inheritance,default)
+                        local list, ok = selective(attribute,content,inheritance,default)
                         if content ~= list then
                             setlist(stack,list)
                         end
-                        done = done or ok
+                        if ok then
+                            done = true
+                        end
                     end
                 else
-                    local list, ok = selective(namespace,attribute,content,inheritance,default)
+                    local list, ok = selective(attribute,content,inheritance,default)
                     if content ~= list then
                         setlist(stack,list)
                     end
-                    done = done or ok
+                    if ok then
+                        done = true
+                    end
                 end
                 -- end nested
             end
         elseif id == rule_code then
--- if subtype(stack) == normal_rule then
-            check = getfield(stack,"width") ~= 0
--- end
+            check = getwidth(stack) ~= 0
         end
 
         if check then
@@ -366,43 +367,55 @@ local function selective(namespace,attribute,head,inheritance,default) -- two at
                 if default and c == inheritance then
                     if current ~= default then
                         local data = nsdata[default]
-                        head = insert_node_before(head,stack,copied(data[nsforced or getattr(stack,nsselector) or nsselector]))
+                        head = insert_node_before(head,stack,copy_node(data[nsforced or getattr(stack,nsselector) or nsselector]))
                         current = default
-                        done = true
+                        if ok then
+                            done = true
+                        end
                     end
                 else
                     local s = getattr(stack,nsselector)
+                 -- local s = nsforced or getattr(stack,nsselector)
                     if current ~= c or current_selector ~= s then
                         local data = nsdata[c]
-                        head = insert_node_before(head,stack,copied(data[nsforced or getattr(stack,nsselector) or nsselector]))
+                        head = insert_node_before(head,stack,copy_node(data[nsforced or s or nsselector]))
+                     -- head = insert_node_before(head,stack,copy_node(data[s or nsselector]))
                         current = c
                         current_selector = s
-                        done = true
+                        if ok then
+                            done = true
+                        end
                     end
                 end
                 if leader then
                     -- begin nested
                     if nstrigger and getattr(stack,nstrigger) then
-                        local outer = getatribute(stack,attribute)
+                        local outer = getattr(stack,attribute)
                         if outer ~= inheritance then
-                            local list, ok = selective(namespace,attribute,leader,inheritance,outer)
+                            local list, ok = selective(attribute,leader,inheritance,outer)
                             if leader ~= list then
                                 setleader(stack,list)
                             end
-                            done = done or ok
+                            if ok then
+                                done = true
+                            end
                         else
-                            local list, ok = selective(namespace,attribute,leader,inheritance,default)
+                            local list, ok = selective(attribute,leader,inheritance,default)
                             if leader ~= list then
                                 setleader(stack,list)
                             end
-                            done = done or ok
+                            if ok then
+                                done = true
+                            end
                         end
                     else
-                        local list, ok = selective(namespace,attribute,leader,inheritance,default)
+                        local list, ok = selective(attribute,leader,inheritance,default)
                         if leader ~= list then
                             setleader(stack,list)
                         end
-                        done = done or ok
+                        if ok then
+                            done = true
+                        end
                     end
                     -- end nested
                     leader = false
@@ -410,12 +423,12 @@ local function selective(namespace,attribute,head,inheritance,default) -- two at
             elseif default and inheritance then
                 if current ~= default then
                     local data = nsdata[default]
-                    head = insert_node_before(head,stack,copied(data[nsforced or getattr(stack,nsselector) or nsselector]))
+                    head    = insert_node_before(head,stack,copy_node(data[nsforced or getattr(stack,nsselector) or nsselector]))
                     current = default
-                    done = true
+                    done    = true
                 end
             elseif current > 0 then
-                head = insert_node_before(head,stack,copied(nsnone))
+                head = insert_node_before(head,stack,copy_node(nsnone))
                 current, current_selector, done = 0, 0, true
             end
             check = false
@@ -426,8 +439,8 @@ local function selective(namespace,attribute,head,inheritance,default) -- two at
 end
 
 states.selective = function(namespace,attribute,head,default)
-    local head, done = selective(namespace,attribute,tonut(head),default)
-    return tonode(head), done
+    local head = selective(attribute,tonut(head),default)
+    return tonode(head), true
 end
 
 -- Ideally the next one should be merged with the previous but keeping it separate is
@@ -439,7 +452,7 @@ end
 -- Todo: make a better stacker. Keep track (in attribute) about nesting level. Not
 -- entirely trivial and a generic solution is nicer (compares to the exporter).
 
-local function stacked(namespace,attribute,head,default) -- no triggering, no inheritance, but list-wise
+local function stacked(attribute,head,default) -- no triggering, no inheritance, but list-wise
     local stack   = head
     local done    = false
     local current = default or 0
@@ -464,77 +477,83 @@ local function stacked(namespace,attribute,head,default) -- no triggering, no in
                     if a and current ~= a and nslistwise[a] then -- viewerlayer / needs checking, see below
                         local p = current
                         current = a
-                        head = insert_node_before(head,stack,copied(nsdata[a]))
-                        local list = stacked(namespace,attribute,content,current) -- two return values
+                        head    = insert_node_before(head,stack,copy_node(nsdata[a]))
+                        local list = stacked(attribute,content,current) -- two return values
                         if content ~= list then
                             setlist(stack,list)
                         end
-                        done = true
-                        head, stack = insert_node_after(head,stack,copied(nsnone))
+                        head, stack = insert_node_after(head,stack,copy_node(nsnone))
                         current = p
+                        done    = true
                     else
-                        local list, ok = stacked(namespace,attribute,content,current)
+                        local list, ok = stacked(attribute,content,current)
                         if content ~= list then
                             setlist(stack,list) -- only if ok
                         end
-                        done = done or ok
+                        if ok then
+                            done = true
+                        end
                     end
                 else
-                    local list, ok = stacked(namespace,attribute,content,current)
+                    local list, ok = stacked(attribute,content,current)
                     if content ~= list then
                         setlist(stack,list) -- only if ok
                     end
-                    done = done or ok
+                    if ok then
+                        done = true
+                    end
                 end
             end
         elseif id == rule_code then
--- if subtype(stack) == normal_rule then
-            check = getfield(stack,"width") ~= 0
--- end
+            check = getwidth(stack) ~= 0
         end
 
         if check then
             local a = getattr(stack,attribute)
             if a then
                 if current ~= a then
-                    head = insert_node_before(head,stack,copied(nsdata[a]))
-                    depth = depth + 1
-                    current, done = a, true
+                    head    = insert_node_before(head,stack,copy_node(nsdata[a]))
+                    depth   = depth + 1
+                    current = a
+                    done    = true
                 end
                 if leader then
-                    local list, ok = stacked(namespace,attribute,content,current)
+                    local list, ok = stacked(attribute,content,current)
                     if leader ~= list then
                         setleader(stack,list) -- only if ok
                     end
-                    done = done or ok
+                    if ok then
+                        done = true
+                    end
                     leader = false
                 end
             elseif default > 0 then
                 --
             elseif current > 0 then
-                head = insert_node_before(head,stack,copied(nsnone))
-                depth = depth - 1
-                current, done = 0, true
+                head    = insert_node_before(head,stack,copy_node(nsnone))
+                depth   = depth - 1
+                current = 0
+                done    = true
             end
             check = false
         end
         stack = getnext(stack)
     end
     while depth > 0 do
-        head = insert_node_after(head,stack,copied(nsnone))
+        head = insert_node_after(head,stack,copy_node(nsnone))
         depth = depth - 1
     end
     return head, done
 end
 
 states.stacked = function(namespace,attribute,head,default)
-    local head, done = stacked(namespace,attribute,tonut(head),default)
+    local head, done = stacked(attribute,tonut(head),default)
     return tonode(head), done
 end
 
 -- experimental
 
-local function stacker(namespace,attribute,head,default) -- no triggering, no inheritance, but list-wise
+local function stacker(attribute,head,default) -- no triggering, no inheritance, but list-wise
 
 --     nsbegin()
     local stacked  = false
@@ -562,31 +581,33 @@ local function stacker(namespace,attribute,head,default) -- no triggering, no in
             elseif nslistwise then
                 local a = getattr(current,attribute)
                 if a and attrib ~= a and nslistwise[a] then -- viewerlayer
-                    head = insert_node_before(head,current,copied(nsdata[a]))
-                    local list = stacker(namespace,attribute,content,a)
+                    head = insert_node_before(head,current,copy_node(nsdata[a]))
+                    local list = stacker(attribute,content,a)
                     if list ~= content then
                         setlist(current,list)
                     end
                     done = true
-                    head, current = insert_node_after(head,current,copied(nsnone))
+                    head, current = insert_node_after(head,current,copy_node(nsnone))
                 else
-                    local list, ok = stacker(namespace,attribute,content,attrib)
+                    local list, ok = stacker(attribute,content,attrib)
                     if content ~= list then
                         setlist(current,list)
                     end
-                    done = done or ok
+                    if ok then
+                        done = true
+                    end
                 end
             else
-                local list, ok = stacker(namespace,attribute,content,default)
+                local list, ok = stacker(attribute,content,default)
                 if list ~= content then
                     setlist(current,list)
                 end
-                done = done or ok
+                if ok then
+                    done = true
+                end
             end
         elseif id == rule_code then
--- if subtype(stack) == normal_rule then
-            check = getfield(current,"width") ~= 0
--- end
+            check = getwidth(current) ~= 0
         end
 
         if check then
@@ -600,11 +621,14 @@ local function stacker(namespace,attribute,head,default) -- no triggering, no in
                 if n then
                     head = insert_node_before(head,current,tonut(n)) -- a
                 end
-                attrib, done = a, true
+                attrib = a
+                done   = true
                 if leader then
                     -- tricky as a leader has to be a list so we cannot inject before
-                    local list, ok = stacker(namespace,attribute,leader,attrib)
-                    done = done or ok
+                    local list, ok = stacker(attribute,leader,attrib)
+                    if ok then
+                        done = true
+                    end
                     leader = false
                 end
             end
@@ -615,21 +639,19 @@ local function stacker(namespace,attribute,head,default) -- no triggering, no in
         current = getnext(current)
     end
 
-if stacked then
-
-    local n = nsend()
-    while n do
-        head = insert_node_after(head,previous,tonut(n))
-        n = nsend()
+    if stacked then
+        local n = nsend()
+        while n do
+            head = insert_node_after(head,previous,tonut(n))
+            n = nsend()
+        end
     end
 
-end
-
     return head, done
 end
 
 states.stacker = function(namespace,attribute,head,default)
-    local head, done = stacker(namespace,attribute,tonut(head),default)
+    local head, done = stacker(attribute,tonut(head),default)
     nsreset()
     return tonode(head), done
 end
