@@ -83,10 +83,6 @@ local pattern_b = spaces * comma^0 * spaces * (key * ((spaces * equal * spaces *
 
 -- "a=1, b=2, c=3, d={a{b,c}d}, e=12345, f=xx{a{b,c}d}xx, g={}" : outer {} removes, leading spaces ignored
 
--- todo: rewrite to fold etc
---
--- parse = lpeg.Cf(lpeg.Carg(1) * lpeg.Cg(key * equal * value) * separator^0,rawset)^0 -- lpeg.match(parse,"...",1,hash)
-
 local hash = { }
 
 local function set(key,value)
@@ -197,6 +193,23 @@ function parsers.settings_to_array(str,strict)
     end
 end
 
+function parsers.settings_to_numbers(str)
+    if not str or str == "" then
+        return { }
+    end
+    if type(str) == "table" then
+        -- fall through
+    elseif find(str,",",1,true) then
+        str = lpegmatch(pattern,str)
+    else
+        return { tonumber(str) }
+    end
+    for i=1,#str do
+        str[i] = tonumber(str[i])
+    end
+    return str
+end
+
 local value     = P(lbrace * C((nobrace + nestedbraces)^0) * rbrace)
                 + C((nestedbraces + nestedbrackets + nestedparents + (1-comma))^0)
 local pattern   = spaces * Ct(value*(separator*value)^0)
@@ -553,14 +566,16 @@ function parsers.rfc4180splitter(specification)
     local field       = escaped + non_escaped + Cc("")
     local record      = Ct(field * (separator * field)^1)
     local headerline  = record * Cp()
-    local wholeblob   = Ct((newline^(specification.strict and -1 or 1) * record)^0)
+    local morerecords = (newline^(specification.strict and -1 or 1) * record)^0
+    local headeryes   = Ct(morerecords)
+    local headernop   = Ct(record * morerecords)
     return function(data,getheader)
         if getheader then
             local header, position = lpegmatch(headerline,data)
-            local data = lpegmatch(wholeblob,data,position)
+            local data = lpegmatch(headeryes,data,position)
             return data, header
         else
-            return lpegmatch(wholeblob,data)
+            return lpegmatch(headernop,data)
         end
     end
 end
@@ -591,10 +606,10 @@ local cardinal    = lpegpatterns.cardinal / tonumber
 local spacers     = lpegpatterns.spacer^0
 local endofstring = lpegpatterns.endofstring
 
-local stepper  = spacers * ( C(cardinal) * ( spacers * S(":-") * spacers * ( C(cardinal) + Cc(true) ) + Cc(false) )
+local stepper  = spacers * ( cardinal * ( spacers * S(":-") * spacers * ( cardinal + Cc(true) ) + Cc(false) )
                * Carg(1) * Carg(2) / ranger * S(", ")^0 )^1
 
-local stepper  = spacers * ( C(cardinal) * ( spacers * S(":-") * spacers * ( C(cardinal) + (P("*") + endofstring) * Cc(true) ) + Cc(false) )
+local stepper  = spacers * ( cardinal * ( spacers * S(":-") * spacers * ( cardinal + (P("*") + endofstring) * Cc(true) ) + Cc(false) )
                * Carg(1) * Carg(2) / ranger * S(", ")^0 )^1 * endofstring -- we're sort of strict (could do without endofstring)
 
 function parsers.stepper(str,n,action)
