@@ -113,8 +113,8 @@ local o_replacements = { -- in main table
     ["$"]              = "\\mmlchar{36}", -- $
     ["%"]              = "\\mmlchar{37}",
     ["&"]              = "\\mmlchar{38}",
-    ["^"]              = "\\mmlchar{94}{}", -- strange, sometimes luatex math sees the char instead of \char
-    ["_"]              = "\\mmlchar{95}{}", -- so we need the {}
+    ["^"]              = "\\mmlchar{94}{}", -- strange, sometimes luatex math sees the char instead of \char so we
+    ["_"]              = "\\mmlchar{95}{}", -- need the {} ... has to do with active mess feedback into scanner
     ["~"]              = "\\mmlchar{126}",
     [" "]              = "",
     ["°"]              = "^\\circ", -- hack
